import ray
import time

@ray.remote
class PairedSendReceive(object):
    """
    Paired send and receive. This mimics the deadlock breaking behavior of MPI.
    """
    msg = ""

    def __init__(self):
        None

    def set_message(self, msg):
        """set the message to be sent"""
        self.msg = msg

    def push_send(self, ooid):
        """initiate message from the sender"""
        # sleep to make sure both actors are started before messaging
        time.sleep(1)
        ### TODO

    def recv(self, msg):
        """remote function called by sender"""
        ### TODO

    def pull_recv(self, ooid):
        """initiate message from the receiver"""
        # sleep to make sure both actors are starte
        time.sleep(1)
        ### TODO

    def send(self):
        """remote function called by receiver"""
        ### TODO

ray.init(num_cpus=4, ignore_reinit_error=True)

# Create Send/Recv objects
sr0 = PairedSendReceive.remote()
sr1 = PairedSendReceive.remote()

# set messages
roid0 = sr0.set_message.remote(f"push_send test from {sr0}")
roid1 = sr1.set_message.remote(f"push_send test from {sr1}")
ray.get(roid0)
ray.get(roid1)

# synchronized (one after the other) push
roid0 = sr0.push_send.remote(sr1)
print(ray.get(roid0))
roid1 = sr1.push_send.remote(sr0)
print(ray.get(roid1))

# set new messages
roid0 = sr0.set_message.remote(f"pull_recv test from {sr0}")
roid1 = sr1.set_message.remote(f"pull_recv test from {sr1}")
ray.get(roid0)
ray.get(roid1)

# synchronized (one after the other) pull
roid0 = sr0.pull_recv.remote(sr1)
print(ray.get(roid0))
roid1 = sr1.pull_recv.remote(sr0)
print(ray.get(roid1))

# set new messages
roid0 = sr0.set_message.remote(f"paired push_send test from {sr0}")
roid1 = sr1.set_message.remote(f"paired pull_recv test from {sr1}")
ray.get(roid0)
ray.get(roid1)

# concurrent paired send and receive
roid0 = sr0.push_send.remote(sr1)
roid1 = sr0.pull_recv.remote(sr1)
print(ray.get(roid0))
print(ray.get(roid1))

ray.shutdown()
